local EGL = select(2, ...)
local Database = EGL:NewModule("Database", "AceEvent-3.0")
local L = EGL.L
local TOTAL_ROWS = 14
local Talents = ElitistGroup.Talents
local roleText = {["tank"] = L["|cffffffff[%d]|r |4Tank:Tanks;"], ["healer"] = L["|cffffffff[%d]|r |4Healer:Healers;"], ["physical"] = L["|cffffffff[%d]|r Physical Damage"], ["magic"] = L["|cffffffff[%d]|r Caster DPS"]}
local userList = {}

function Database:OnInitialize()
	self:RegisterMessage("EGL_UI_LOADED", "CreateUI")
	self:RegisterMessage("EG_DATA_UPDATED", function(event, type, playerID)
		if( Database.containerFrame and ( not userList[playerID] or type == "talents" or type == "notes" ) ) then
			if( Database.containerFrame:IsVisible() ) then
				Database:BuildUser(playerID, ElitistGroup.userData[playerID], true)
			else
				Database.rebuildDatabase = true
			end
		end
	end)
end

local function sortAverage(a, b)
	if( a.average ~= b.average ) then
		return a.average > b.average
	end
	
	return a.name < b.name
end
	
function Database:BuildUser(playerID, userData, update)
	local specType = Talents.talentText[ElitistGroup:GetPlayerSpec(userData.classToken, userData)] or "unknown"
	local roleType = specType == Talents.TANK and "tank" or specType == Talents.HEALER and "healer" or specType == Talents.RANGE_DAMAGE and "physical" or specType == Talents.MELEE_DAMAGE and "physical" or specType == Talents.CASTER_DAMAGE and "magic"
	if( not roleType ) then return end

	local user = userList[playerID] or {}
	user.playerID = playerID
	user.classToken = userData.classToken
	user.level = userData.level
	user.name = userData.name
	user.server = userData.server
	user.roleType = roleType
	user.average = 0
	
	-- Figure out note rating
	local total = 0
	if( userData.notes[ElitistGroup.playerID] ) then
		total = total + 1
		user.average = userData.notes[ElitistGroup.playerID].rating
		user.comment = userData.notes[ElitistGroup.playerID].comment
	end

	if( not EGL.db.playerOnly ) then
		for noteID, note in pairs(userData.notes) do
			if( noteID ~= ElitistGroup.playerID ) then
				total = total + 1
				user.average = user.average + note.rating
			end
		end
		
		user.average = total == 0 and -1 or user.average / total
	end
	
	if( not userList[playerID] ) then
		userList[playerID] = user
		table.insert(userList, user)
	end
	
	if( update ) then
		table.sort(userList, sortAverage)
		self:Update(self.roleFrames[roleType])
	end
end

local playerRealm = GetRealmName()
function Database:Build()
	if( not self.rebuildDatabase ) then return end
	self.rebuildDatabase = nil
	
	for playerID in pairs(ElitistGroup.db.faction.users) do
		local userData = ElitistGroup.userData[playerID]
		-- As this is specialized for forming groups, we can restrict it to same-server for the time being
		if( not userData.pruned and userData.classToken and userData.server == playerRealm ) then
			self:BuildUser(playerID, userData)
		end
	end
	
	table.sort(userList, sortAverage)
end

function Database:Update(frame)
	self:Build()

	local totalPlayers, rowID = 0, 0
	for i=1, #(userList) do
		local summaryData = userList[i]
		if( summaryData.roleType == frame.viewType ) then
			totalPlayers = totalPlayers + 1
			
			if( totalPlayers >= frame.scroll.offset ) then
				rowID = rowID + 1

				local row = frame.rows[rowID]
				local classColor = RAID_CLASS_COLORS[summaryData.classToken]
				row.fontString:SetTextColor(classColor.r, classColor.g, classColor.b)
				
				if( summaryData.average == -1 ) then
					row:SetText(summaryData.name)
				else
					row:SetFormattedText("|cffffffff%d.|r %s", summaryData.average, summaryData.name)
				end
				
				if( rowID >= TOTAL_ROWS ) then break end
			end
		end
	end
	
	local rowWidth = frame:GetWidth() - (totalPlayers > TOTAL_ROWS and 24 or 4)
	for id, row in pairs(frame.rows) do
		if( id <= rowID ) then
			row:SetWidth(rowWidth)
			row:Show()
		else
			row:Hide()
		end
	end
	
	frame.scroll:Update(totalPlayers)
	frame.headerText:SetFormattedText(roleText[frame.viewType], totalPlayers)
end

function Database:CreateUI(event, parent)
	self.rebuildDatabase = true
	
	local backdrop = {bgFile = "Interface\\ChatFrame\\ChatFrameBackground", edgeFile = "Interface\\ChatFrame\\ChatFrameBackground", edgeSize = 1}
	local containerFrame = CreateFrame("Frame", nil, parent)
	containerFrame:SetAllPoints(parent)
	containerFrame:SetSize(1, 1)
	containerFrame:Hide()
	containerFrame:SetScript("OnShow", function()
		for i=1, #(Database.roleFrames) do
			Database:Update(Database.roleFrames[i])
		end
	end)

	local roleFrames = {}

	local roles = {"tank", "healer", "physical", "magic"}
	for id, role in pairs(roles) do
		-- Create the main container frame
		local frame = CreateFrame("Frame", nil, containerFrame)   
		frame:SetBackdrop(backdrop)
		frame:SetBackdropBorderColor(0.60, 0.60, 0.60, 1)
		frame:SetBackdropColor(0, 0, 0, 0)
		frame:SetWidth(155)
		frame:SetHeight(300)
		frame.viewType = role
		
		frame.headerText = frame:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
		frame.headerText:SetPoint("BOTTOMLEFT", frame, "TOPLEFT", 0, 5)

		if( id > 1 ) then
			frame:SetPoint("TOPLEFT", roleFrames[id - 1], "TOPRIGHT", 4, 0)
		else
			frame:SetPoint("TOPLEFT", containerFrame, "TOPLEFT", 12, -40)
		end

		roleFrames[role] = frame
		table.insert(roleFrames, frame)

		ElitistGroup.Widgets:CreateScrollFrame(frame, TOTAL_ROWS, nil, self, "Update")
		frame.scroll:Update(TOTAL_ROWS + 20)
		frame.scroll:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, -2)
		frame.scroll:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -24, 33)
	
		local highlightR, highlightG, highlightB = GameFontNormal:GetTextColor()
		local function OnEnter(self)
			self.startR, self.startG, self.startB = self.fontString:GetTextColor()
			self.fontString:SetTextColor(highlightR, highlightG, highlightB)
			
			ElitistGroup.Widgets.OnEnter(self)
		end
		
		local function OnLeave(self)
			self.fontString:SetTextColor(self.startR, self.startG, self.startB)
			ElitistGroup.Widgets.OnLeave(self)
		end
		
		-- Create all of the rows
		frame.rows = {}
		for i=1, TOTAL_ROWS do
			local button = CreateFrame("Button", nil, frame)
			button:SetNormalFontObject(GameFontNormal)
			button:SetHighlightFontObject(GameFontHighlight)
			button:SetScript("OnEnter", OnEnter)
			button:SetScript("OnLeave", OnLeave)
			button:SetText("*")
			button:SetHeight(20)
			button:SetWidth(frame:GetWidth() - 24)

			local fontString = button:GetFontString()
			fontString:SetJustifyH("LEFT")
			fontString:SetJustifyV("CENTER")
			fontString:SetPoint("LEFT", button, "LEFT")
			fontString:SetPoint("RIGHT", button, "RIGHT")
			button.fontString = fontString
			
			if( i > 1 ) then
				button:SetPoint("TOPLEFT", frame.rows[i - 1], "BOTTOMLEFT", 0, -1)
			else
				button:SetPoint("TOPLEFT", frame, "TOPLEFT", 3, -2)
			end

			frame.rows[i] = button
		end
	end
	
	self.roleFrames = roleFrames
	self.containerFrame = containerFrame
	
	EGL.modules.UI:CreateTab(containerFrame, L["Extended Database"], 3, "database")
end
